svg circle { 
    stroke: black;
    stroke-width: 1px;
    fill: grey;
    opacity: 0.8;
}
svg text {
    font-family: sans-serif;
    fill: black;
}
svg line {
    stroke: black;
    stroke-width: 5px;
}
svg path {
    stroke: blue;
    fill: none;
}

div.tooltip {
    position: absolute;
    background-color: #F0F0F0;
    border-width: 1px;
    border-style: solid;
    border-color: LightGray;
    font-size: 10px;
    padding: 0.6em;
    opacity: 0;
}

svg {display:block;}

div.tooltip dl {margin: 0px;}
div.tooltip dt {color: DimGray;}
